﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.Core;
using System.Collections.Specialized;
using System.Collections;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Drawing;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;

namespace Navigator
{
    public class NavPoint
    {
        public UInt32 Line;
        public UInt32 Station;
        public Int32 Lat;
        public Int32 Lon;
        public bool bSelected;
        public string GetLabel()
        {
            return string.Format("{0:0.##} \n{1:0.##}", (double)Line / 100, (double)Station / 100);
        }
    }
    class PointsFeatureLayer : InMemoryFeatureLayer
    {
        public const double constLatSec = 1.50874257088;
        public const double constLonSec = 3.01748514176;
        public PointsPointStyle pointStyle = null;
        public PointsTextStyle textStyle = null;


        // vars from settings saved in registry
        Color ReceiverColor = Color.Green, SourceColor = Color.Red, SweepColor = Color.Yellow;
        bool bLightTheme = false; 
        Color LabelColor = Color.White; 
        int MapFontSize = 9;
        bool bViewLabels = true;
        bool bViewSources = true;
        /////////
        public PointsFeatureLayer()
        {
            Open();
            Columns.Add(new FeatureSourceColumn("Type"));
            Columns.Add(new FeatureSourceColumn("Label"));
            Close();
            pointStyle = new PointsPointStyle( ReceiverColor, SourceColor, SweepColor);
            textStyle = new PointsTextStyle(bLightTheme, LabelColor, MapFontSize);
            ZoomLevelSet.ZoomLevel01.DefaultTextStyle = textStyle;
            ZoomLevelSet.ZoomLevel01.DefaultPointStyle = pointStyle;
            ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
        }

        public bool LoadPoints()
        {
            List<NavPoint> NavPoints = new List<NavPoint>();
            string sFileName = Application.StartupPath + "\\lineprep.xml";
            XmlSerializer NavPointsDeserializer = new XmlSerializer(typeof(List<NavPoint>));
              using (TextReader NavPointTextReader = new StreamReader(sFileName))
                NavPoints = (List<NavPoint>)NavPointsDeserializer.Deserialize(NavPointTextReader);
            if (NavPoints.Count == 0) return true;
            foreach (NavPoint aNavPoint in NavPoints)
            {
                try
                {
                    double Lon = aNavPoint.Lon * constLonSec / 3600 / 10000;
                    double Lat = aNavPoint.Lat * constLatSec / 3600 / 10000;
                    Feature aFeature = new Feature(Lon, Lat);
                    aFeature.ColumnValues["Type"] = "Source";
                    aFeature.ColumnValues["Label"] = aNavPoint.GetLabel();
                    InternalFeatures.Add(aFeature);
                }
                catch { }
            }
            return true;
        }

        public void SetSourceVisibility(bool IsVisible)
        {
            pointStyle.ViewSources = IsVisible;
            textStyle.ViewSourceLabels = IsVisible && bViewLabels;
        }

        public void SetLabelsVisibility(bool IsVisible)
        {
            textStyle.ViewSourceLabels = IsVisible && bViewSources;
        }

    }

}
