﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using ThinkGeo.Core;
using System.IO;
using System.Drawing.Imaging;

namespace Navigator
{
    public class PointsPointStyle : PointStyle
    {
        GeoImage ReceiverImage = null;
        GeoImage SourceImage = null;
        GeoImage CompletedPointImage = null;
        GeoImage SweepPointImage = null;
        GeoImage DeployedPointImage = null;
        GeoImage LineNavigationPointImage = null;

        bool viewReceivers = true;
        public bool ViewReceivers
        {
            get { return viewReceivers; }
            set { viewReceivers = value; }
        }
        bool viewSources = true;
        public bool ViewSources
        {
            get { return viewSources; }
            set { viewSources = value; }
        }
        bool viewSweepPoints = true;
        public bool ViewSweepPoints
        {
            get { return viewSweepPoints; }
            set { viewSweepPoints = value; }
        }

        public PointsPointStyle(Color ReceiverColor, Color SourceColor, Color SweepColor)
        {
            Bitmap ReceiverBmp = new Bitmap(6, 6);
            using (Graphics graphics = Graphics.FromImage(ReceiverBmp))
            {
                graphics.CompositingMode = CompositingMode.SourceCopy;
                graphics.CompositingQuality = CompositingQuality.HighQuality;
                graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;

                using (Brush brush = GetBrush(ReceiverColor))
                    graphics.FillEllipse(brush, 0, 0, 5, 5);
            }
            ReceiverImage = new GeoImage((byte[])new ImageConverter().ConvertTo(ReceiverBmp, typeof(byte[])));
            Bitmap SourceBmp = new Bitmap(6, 6);
            using (Graphics graphics = Graphics.FromImage(SourceBmp))
            {
                graphics.CompositingMode = CompositingMode.SourceCopy;
                graphics.CompositingQuality = CompositingQuality.HighQuality;
                graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
                using (Brush brush = GetBrush(SourceColor))
                    graphics.FillEllipse(brush, 0, 0, 5, 5);
            }
            SourceImage = new GeoImage((byte[])new ImageConverter().ConvertTo(SourceBmp, typeof(byte[])));
            Bitmap CompletedPointBmp = new Bitmap(12, 12);
            using (Graphics graphics = Graphics.FromImage(CompletedPointBmp))
            {
                graphics.CompositingMode = CompositingMode.SourceCopy;
                graphics.CompositingQuality = CompositingQuality.HighQuality;
                graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
                using (Brush brush = GetBrush(Color.Red))
                    graphics.FillEllipse(brush, 0, 0, 11, 11);
                using (Brush brush = GetBrush(Color.Yellow))
                    graphics.FillEllipse(brush, 1, 1, 9, 9);
            }
            CompletedPointImage = new GeoImage((byte[])new ImageConverter().ConvertTo(CompletedPointBmp, typeof(byte[])));
            Bitmap LineNavigationPointBmp = new Bitmap(11, 11);
            using (Graphics graphics = Graphics.FromImage(LineNavigationPointBmp))
            {
                graphics.CompositingMode = CompositingMode.SourceCopy;
                graphics.CompositingQuality = CompositingQuality.HighQuality;
                graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
                using (Brush brush = GetBrush(Color.Blue))
                    graphics.FillEllipse(brush, 0, 0, 10, 10);
                using (Brush brush = GetBrush(Color.Orange))
                    graphics.FillEllipse(brush, 2, 2, 6, 6);
            }
            LineNavigationPointImage = new GeoImage((byte[])new ImageConverter().ConvertTo(LineNavigationPointBmp, typeof(byte[])));
            Bitmap SweepPointBmp = new Bitmap(12, 12);
            using (Graphics graphics = Graphics.FromImage(SweepPointBmp))
            {
                graphics.CompositingMode = CompositingMode.SourceCopy;
                graphics.CompositingQuality = CompositingQuality.HighQuality;
                graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
                Point[] pts = { new Point(6, 0), new Point(0, 6), new Point(6, 11), new Point(11, 6) };
                using (Brush brush = GetBrush(SweepColor))
                    graphics.FillPolygon(brush, pts);
            }
            SweepPointImage = new GeoImage((byte[])new ImageConverter().ConvertTo(SweepPointBmp, typeof(byte[])));
            Bitmap DeployedPointBmp = new Bitmap(6, 6);
            using (Graphics graphics = Graphics.FromImage(DeployedPointBmp))
            {
                graphics.CompositingMode = CompositingMode.SourceCopy;
                graphics.CompositingQuality = CompositingQuality.HighQuality;
                graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;

                using (Brush brush = GetBrush(Color.SkyBlue))
                    graphics.FillEllipse(brush, 0, 0, 5, 5);
            }
            DeployedPointImage = new GeoImage((byte[])new ImageConverter().ConvertTo(DeployedPointBmp, typeof(byte[])));
        }

        int VarFromSettings_MaxDistanceFromSource = 15;
        public int GetDestinationDiameterInPixels(GeoCanvas aCanvas, PointShape TargetPointShape)
        {
            PointShape TempPoint = (PointShape)BaseShape.TranslateByOffset(TargetPointShape,
             2 * VarFromSettings_MaxDistanceFromSource, 0, GeographyUnit.DecimalDegree, DistanceUnit.Meter);
            int DestinationDiameter = (int)MapUtil.GetScreenDistanceBetweenTwoWorldPoints(
                 aCanvas.CurrentWorldExtent, TargetPointShape, TempPoint, aCanvas.Width, aCanvas.Height);
            if (DestinationDiameter < 18) DestinationDiameter = 18;
            return DestinationDiameter;
        }

        private System.Drawing.Brush GetBrush(Color brushColor)
        {
            Color color = Color.FromArgb(brushColor.A, brushColor.R, brushColor.G, brushColor.B);
            return new SolidBrush(color);
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, 
            Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            RectangleShape currentExtent = canvas.CurrentWorldExtent;
            double MinX = currentExtent.UpperLeftPoint.X; double MaxX = currentExtent.UpperRightPoint.X;
            double MaxY = currentExtent.UpperLeftPoint.Y; double MinY = currentExtent.LowerLeftPoint.Y;
            if (MinX < -180 || MaxX > 180 || MinY < -90 || MaxY > 90)
                return;

            RectangleShape ClientRect = canvas.CurrentWorldExtent;
            foreach (Feature aFeature in features)
            {
                PointShape aPointShape = (PointShape)aFeature.GetShape();
                if (aPointShape.X < ClientRect.UpperLeftPoint.X || aPointShape.X > ClientRect.UpperRightPoint.X
                || aPointShape.Y < ClientRect.LowerLeftPoint.Y || aPointShape.Y > ClientRect.UpperLeftPoint.Y) continue;
                bool bDrawFeature = (((aFeature.ColumnValues["Type"] == "Receiver") && viewReceivers)
                    || ((aFeature.ColumnValues["Type"] == "Source") && viewSources)
                    || aFeature.ColumnValues["Type"] == "PSSDestination"
                    || (aFeature.ColumnValues["Type"] == "SweepPosition" && viewSweepPoints)
                    || aFeature.ColumnValues["Type"] == "DeployedPoint"
                    || aFeature.ColumnValues["Type"] == "CompletedPoint"
                    || aFeature.ColumnValues["Type"] == "NavPoint");
                if (bDrawFeature)
                {
                    //base.DrawCore(new Feature[1] { aFeature }, canvas, labelsInThisLayer, labelsInAllLayers);
                    if (aFeature.ColumnValues["Type"] == "PSSDestination")
                    {
                        int DiameterInPixels = GetDestinationDiameterInPixels(canvas, aPointShape);
                        Bitmap TargetPointBmp = new Bitmap(DiameterInPixels, DiameterInPixels);
                        using (Graphics graphics = Graphics.FromImage(TargetPointBmp))
                        {
                            graphics.SmoothingMode = SmoothingMode.AntiAlias;
                            graphics.CompositingMode = CompositingMode.SourceCopy;
                            graphics.CompositingQuality = CompositingQuality.HighQuality;
                            graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
                            Color TargetColor = Color.FromArgb(100, Color.Yellow);
                            using (Brush brush = GetBrush(TargetColor))
                                graphics.FillEllipse(brush, 0, 0, DiameterInPixels - 1, DiameterInPixels - 1);
                            using (Pen pen = new Pen(Color.Orange, 3))
                                graphics.DrawEllipse(pen, 2, 2, DiameterInPixels - 5, DiameterInPixels - 5);
                            using (Brush brush = GetBrush(Color.Red))
                                graphics.FillEllipse(brush, DiameterInPixels / 2 - 2, DiameterInPixels / 2 - 2, 5, 5);
                        }
                        GeoImage TargetPointImage = new GeoImage((byte[])new ImageConverter().ConvertTo(TargetPointBmp, typeof(byte[])));
                        canvas.DrawWorldImageWithoutScaling(TargetPointImage, aPointShape.X, aPointShape.Y, DrawingLevel.LevelOne);
                    }
                    else
                    {
                        GeoImage DrawImage;
                        if (aFeature.ColumnValues["Type"] == "Receiver") DrawImage = ReceiverImage;
                        else if (aFeature.ColumnValues["Type"] == "CompletedPoint") DrawImage = CompletedPointImage;
                        else if (aFeature.ColumnValues["Type"] == "SweepPosition") DrawImage = SweepPointImage;
                        else if (aFeature.ColumnValues["Type"] == "DeployedPoint") DrawImage = DeployedPointImage;
                        else if (aFeature.ColumnValues["Type"] == "NavPoint") DrawImage = LineNavigationPointImage;
                        else DrawImage = SourceImage;
                        canvas.DrawWorldImageWithoutScaling(DrawImage, aPointShape.X, aPointShape.Y, DrawingLevel.LevelFour);
                    }
                }
            }
        }

        protected override Collection<string> GetRequiredColumnNamesCore()
        {
            Collection<string> columns = new Collection<string>();
            if (!columns.Contains("Type")) columns.Add("Type");
            return columns;
        }
    }

    public class SweepPointStyle : PointStyle
    {
        GeoImage SweepPointImage = null;
        bool viewSweepPoints = true;
        public bool ViewSweepPoints
        {
            get { return viewSweepPoints; }
            set { viewSweepPoints = value; }
        }
        public SweepPointStyle(Color SweepColor) 
        {
            Bitmap SweepPointBmp = new Bitmap(12, 12);
            using (Graphics graphics = Graphics.FromImage(SweepPointBmp))
            {
                graphics.CompositingMode = CompositingMode.SourceCopy;
                graphics.CompositingQuality = CompositingQuality.HighQuality;
                graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
                Point[] pts = { new Point(6, 0), new Point(0, 6), new Point(6, 11), new Point(11, 6) };
                graphics.FillPolygon(new SolidBrush(SweepColor), pts);
            }
            SweepPointImage = ConvertToGeoImage(SweepPointBmp);
        }
        private GeoImage ConvertToGeoImage(Bitmap Bmp)
        {
            Stream stream = new MemoryStream();
            Bmp.Save(stream, ImageFormat.Tiff);
            return new GeoImage(stream);
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas,
              Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            RectangleShape currentExtent = canvas.CurrentWorldExtent;
            double MinX = currentExtent.UpperLeftPoint.X; double MaxX = currentExtent.UpperRightPoint.X;
            double MaxY = currentExtent.UpperLeftPoint.Y; double MinY = currentExtent.LowerLeftPoint.Y;
            if (MinX < -180 || MaxX > 180 || MinY < -90 || MaxY > 90)
                return;

            if (!viewSweepPoints) return;
            foreach (Feature aFeature in features)
            {
                PointShape pointShape = (PointShape)aFeature.GetShape();
                canvas.DrawWorldImageWithoutScaling(SweepPointImage, pointShape.X, pointShape.Y, DrawingLevel.LevelFour);
            }
        }
    }

}
