﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.Core;
using System.Drawing;

namespace Navigator
{
    class PointsTextStyle : TextStyle
    {
        private bool viewReceiverLabels = true;
        public bool ViewReceiverLabels
        {
            get { return viewReceiverLabels; }
            set { viewReceiverLabels = value; }
        }
        bool viewSourceLabels = true;
        public bool ViewSourceLabels
        {
            get { return viewSourceLabels; }
            set { viewSourceLabels = value; }
        }

        private bool suppressLabels = false;
        public bool SuppressLabels
        {
            get { return suppressLabels; }
            set { suppressLabels = value; }
        }

         public PointsTextStyle(bool bLightTheme, Color LabelColor, float FontSize) 
         {
             TextColumnName = "Label";
             Font = new GeoFont("Tahoma", FontSize, DrawingFontStyles.Bold);
             TextBrush = new GeoSolidBrush(new GeoColor(LabelColor.R, LabelColor.G, LabelColor.B));
             HaloPen = bLightTheme ? new GeoPen(GeoColors.White, 1) : new GeoPen(GeoColors.Black, 1);
             SuppressLabels = true;
             DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
             GridSize = 7;
             OverlappingRule = LabelOverlappingRule.AllowOverlapping;
         }

        public class FieldRect
        {
            public RectangleShape RectShape;
            public int NumberOfLabels;
            public List<Feature> FeatureList = new List<Feature>(); 
            public FieldRect(RectangleShape RectShape)
            {
                this.RectShape = RectShape;
            }
        }

        List<FieldRect> FieldRectList = new List<FieldRect>();

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas,
           Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            int NumFeatures = 0;
            foreach (Feature aFeature in features) NumFeatures++; 
            canvas.DrawingQuality = DrawingQuality.HighSpeed;
            RectangleShape ExtentRect = new RectangleShape(canvas.CurrentWorldExtent.UpperLeftPoint, canvas.CurrentWorldExtent.LowerRightPoint);
            double ShiftScaleWidth = ExtentRect.Width / 5; double ShiftScaleHeight = ExtentRect.Height / 5;
            /*
            ExtentRect.ScaleDown(25);
            if (NumFeatures > 700)
            {  // Draw features only which presents in FieldRect from FieldList
                OverlappingRule = LabelOverlappingRule.AllowOverlapping;
                CreateChessFieldsList(ExtentRect);
                // Draw general Field and chess Fields on map
                //canvas.DrawArea(ExtentRect, new GeoSolidBrush(GeoColor.StandardColors.DarkGreen), DrawingLevel.LevelOne);
                //foreach (FieldRect aField in FieldRectList)
                //   canvas.DrawArea(aField.RectShape, new GeoSolidBrush(GeoColor.StandardColors.Black), DrawingLevel.LevelTwo);

                int NumLabelsPerField = 1;
                foreach (Feature aFeature in features)
                {
                    PointShape FeaturePointShape = (PointShape)aFeature.GetShape();
                    bool bDrawFeature = (((aFeature.ColumnValues["Type"] == "Receiver") && viewReceiverLabels)
                      || ((aFeature.ColumnValues["Type"] == "Source") && viewSourceLabels)
                      || aFeature.ColumnValues["Type"] == "DeployedPoint"
                      || aFeature.ColumnValues["Type"] == "CompletedPoint");
                      //|| aFeature.ColumnValues["Type"] == "NavPoint");
                    if (bDrawFeature)

                    {
                        foreach (FieldRect aField in FieldRectList)
                        {
                            if (aField.NumberOfLabels < NumLabelsPerField)
                                if (FieldContainsPointShape(aField.RectShape, FeaturePointShape))
                                {
                                    aField.NumberOfLabels++;
                                    aField.FeatureList.Add(aFeature);
                                    //base.DrawCore(new Feature[1] { aFeature }, canvas, labelsInThisLayer, labelsInAllLayers);
                                }
                        }
                    }
                }
                foreach (FieldRect aField in FieldRectList)
                   if (aField.NumberOfLabels > 0)
                      base.DrawCore(aField.FeatureList.ToArray(), canvas, labelsInThisLayer, labelsInAllLayers);
            }
            else
            */
            { // Draw all the feature which presents in ExtentRect
                OverlappingRule = LabelOverlappingRule.NoOverlapping;
                foreach (Feature aFeature in features)
                {
                    PointShape FeaturePointShape = (PointShape)aFeature.GetShape();
                    bool bDrawFeature = (((aFeature.ColumnValues["Type"] == "Receiver") && viewReceiverLabels)
                      || ((aFeature.ColumnValues["Type"] == "Source") && viewSourceLabels)
                      || aFeature.ColumnValues["Type"] == "DeployedPoint"
                      || aFeature.ColumnValues["Type"] == "CompletedPoint");
                      //|| aFeature.ColumnValues["Type"] == "NavPoint");

                    if (bDrawFeature)
                    {
                        if (FieldContainsPointShape(ExtentRect, FeaturePointShape))
                            base.DrawCore(new Feature[1] { aFeature }, canvas, labelsInThisLayer, labelsInAllLayers);
                    }
                }
            }
        }

        /*
         protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas,
            Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            int NumFeatures = 0;
            foreach (Feature aFeature in features) NumFeatures++; 
            canvas.DrawingQuality = DrawingQuality.HighSpeed;
            RectangleShape ExtentRect = new RectangleShape(canvas.CurrentWorldExtent.UpperLeftPoint, canvas.CurrentWorldExtent.LowerRightPoint);
            double ShiftScaleWidth = ExtentRect.Width / 5; double ShiftScaleHeight = ExtentRect.Height / 5;
            ExtentRect.ScaleDown(25);
            if (NumFeatures > 700)
            {  // Draw features only which presents in FieldRect from FieldList
                OverlappingRule = LabelOverlappingRule.AllowOverlapping;
                CreateChessFieldsList(ExtentRect);
                 // Draw general Field and chess Fields on map
                //canvas.DrawArea(ExtentRect, new GeoSolidBrush(GeoColor.StandardColors.DarkGreen), DrawingLevel.LevelOne);
                //foreach (FieldRect aField in FieldRectList)
                //   canvas.DrawArea(aField.RectShape, new GeoSolidBrush(GeoColor.StandardColors.Black), DrawingLevel.LevelTwo);
               

                int NumLabelsPerField = 1;
                foreach (Feature aFeature in features)
                {
                    PointShape FeaturePointShape = (PointShape)aFeature.GetShape();
                    bool bDrawFeature = (((aFeature.ColumnValues["Type"] == "Receiver") && viewReceiverLabels)
                      || ((aFeature.ColumnValues["Type"] == "Source") && viewSourceLabels)
                      || aFeature.ColumnValues["Type"] == "DeployedPoint"
                      || aFeature.ColumnValues["Type"] == "CompletedPoint");
                    if (bDrawFeature)
                    {
                        foreach (FieldRect aField in FieldRectList)
                        {
                            if (aField.NumberOfLabels < NumLabelsPerField)
                                if (FieldContainsPointShape(aField.RectShape, FeaturePointShape))
                                {
                                    aField.NumberOfLabels++;
                                    base.DrawCore(new Feature[1] { aFeature }, canvas, labelsInThisLayer, labelsInAllLayers);
                                }
                        }
                    }
                }
            }
            else
            { // Draw all the feature which presents in ExtentRect
                OverlappingRule = LabelOverlappingRule.NoOverlapping;
                foreach (Feature aFeature in features)
                {
                    PointShape FeaturePointShape = (PointShape)aFeature.GetShape();
                    bool bDrawFeature = (((aFeature.ColumnValues["Type"] == "Receiver") && viewReceiverLabels)
                      || ((aFeature.ColumnValues["Type"] == "Source") && viewSourceLabels)
                      || aFeature.ColumnValues["Type"] == "DeployedPoint" 
                      || aFeature.ColumnValues["Type"] == "CompletedPoint");
                    if (bDrawFeature)
                    {
                        if (FieldContainsPointShape(ExtentRect, FeaturePointShape))
                            base.DrawCore(new Feature[1] { aFeature }, canvas, labelsInThisLayer, labelsInAllLayers);
                    }
                }
            }
        }
        */

         void CreateChessFieldsList(RectangleShape ExtentRect)
         {
             FieldRectList.Clear();
             int NumCell = 30;
             double CellWidth = ExtentRect.Width / (double)(NumCell);
             double CellHeight = ExtentRect.Height / (double)(NumCell);
             int ShiftCol = 0;
             for (int Row = 0; Row < NumCell; Row ++)
             { // Calc chess fields
                 ShiftCol = ShiftCol + 1 > 3 ? 0 : ShiftCol + 1;
                 for (int Col = ShiftCol; Col < NumCell; Col += 4)
                 {
                     PointShape FieldUpperLeftPoint = new PointShape(ExtentRect.UpperLeftPoint.X + (double)Row * CellWidth, ExtentRect.UpperLeftPoint.Y - (double)(Col + 1) * CellHeight);
                     PointShape FieldLowerRightPoint = new PointShape(FieldUpperLeftPoint.X + CellWidth, FieldUpperLeftPoint.Y + CellHeight);
                     RectangleShape aRectShape = new RectangleShape(FieldUpperLeftPoint.X, FieldLowerRightPoint.Y, FieldLowerRightPoint.X, FieldUpperLeftPoint.Y);
                     FieldRectList.Add(new FieldRect(aRectShape));
                 }
             }

             /*
             bool ShiftCol = true;
             for (int Row = 0; Row < NumCell; Row += 2)
             { // Calc chess fields
                 ShiftCol = !ShiftCol;
                 for (int Col = ShiftCol ? 1 : 0; Col < NumCell; Col += 2)
                 {
                     PointShape FieldUpperLeftPoint = new PointShape(ExtentRect.UpperLeftPoint.X + (double)Row * CellWidth, ExtentRect.UpperLeftPoint.Y - (double)Col * CellHeight);
                     PointShape FieldLowerRightPoint = new PointShape(FieldUpperLeftPoint.X + CellWidth, FieldUpperLeftPoint.Y + CellHeight);
                     RectangleShape aRectShape = new RectangleShape(FieldUpperLeftPoint.X, FieldLowerRightPoint.Y, FieldLowerRightPoint.X, FieldUpperLeftPoint.Y);
                     FieldRectList.Add(new FieldRect(aRectShape));
                 }
             }
              */
         }

         bool FieldContainsPointShape(RectangleShape aRect, PointShape aPoint)
         {
             if (aPoint.X > aRect.UpperLeftPoint.X && aPoint.X < aRect.LowerRightPoint.X &&
                 aPoint.Y < aRect.UpperLeftPoint.Y && aPoint.Y > aRect.LowerRightPoint.Y)
                 return true;
             else return false;
         }
        protected override System.Collections.ObjectModel.Collection<string> GetRequiredColumnNamesCore()
        {
            Collection<string> columns = new Collection<string>();
            if (!columns.Contains(TextColumnName)) columns.Add(TextColumnName);
            return columns;
        }
    }
}
